package com.dbnt.sims.fipTarget.repository;


import com.dbnt.sims.fipTarget.model.PartWorkFile;


import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;



public interface PartWorkFileRepository extends JpaRepository<PartWorkFile, PartWorkFile.PartWorkFileId> {

	PartWorkFile findTopByPwSeqAndPiSeqOrderByFileSeqDesc(Integer piSeq, Integer piSeq2);

	List<PartWorkFile> findByPwSeqAndPiSeqOrderByFileSeqAsc(Integer pwSeq, Integer piSeq);

	List<PartWorkFile> findByPwSeqAndPiSeq(Integer pwSeq, Integer piSeq);

	void deleteByPwSeqAndPiSeq(Integer pwSeq, Integer piSeq);

	List<PartWorkFile> findByPiSeq(Integer piSeq);













}
